@@ -58,6 +58,7 @@ import json
 import logging
 import os
 import stat
+import copy
 from typing import Optional
 
 import requests
@@ -92,7 +93,7 @@ class NuvlaResourceOperationNotAvailable(KeyError):
 
 
 def _request_debug(method, endpoint, params=None, doc=None,
-                   data=None, headers: Optional[dict]=None):
+                   data=None, headers: Optional[dict] = None):
     # TODO: Use logging library.
     print('::: HTTP Request -> Response :::')
     print('>>> Request')
@@ -154,7 +155,9 @@ class SessionStore(requests.Session):
                 self.cookies.clear_expired_cookies()
 
     def need_to_login(self, accessed_url, status_code):
-        return self.reauthenticate and status_code in [401, 403] and accessed_url != self.session_base_url
+        return self.reauthenticate \
+               and status_code in [401, 403] \
+               and accessed_url != self.session_base_url
 
     def _request(self, *args, **kwargs):
         kwargs.setdefault('timeout', DEFAULT_TIMEOUT)
@@ -234,8 +237,9 @@ def to_login_params(creds):
 class Api(object):
     """ This class is a Python wrapper&helper of the native Nuvla REST API"""
 
-    def __init__(self, endpoint=DEFAULT_ENDPOINT, insecure=False, persist_cookie=True, cookie_file=None,
-                 reauthenticate=False, login_creds=None, authn_header=None, debug=False):
+    def __init__(self, endpoint=DEFAULT_ENDPOINT, insecure=False, persist_cookie=True,
+                 cookie_file=None, reauthenticate=False, login_creds=None, authn_header=None,
+                 debug=False):
         """
         :param endpoint: Nuvla endpoint (https://nuvla.io).
         :param insecure: Don't check server certificate or you are using a http connection.
@@ -500,6 +504,30 @@ class Api(object):
                 self._cimi_request('DELETE', resource_type,
                                    data={'filter': filter}, headers={'bulk': 'yes'}))
 
+    def operation_bulk(self, resource_type, filter, operation, data=None) -> CimiResponse:
+        """ Bulk delete CIMI resources of the given type (Collection).
+
+        :param      resource_type: Type of the resource (Collection name).
+        :type       resource_type: str
+
+        :param      Non-empty CIMI filter.
+        :type       filter: str
+
+        :param      data: The data to serialize into JSON
+        :type       data: dict
+
+        :return:    A CimiResponse object which should contain the attributes 'status', 'resource-id', 'message' and 'location'
+        :rtype:     CimiResponse
+        """
+        if not isinstance(filter, str) or len(filter) == 0:
+            raise NuvlaError("'filter' must be a non-empty string.")
+        else:
+            data_with_filter = copy.deepcopy(data)
+            data_with_filter['filter'] = filter
+            return CimiResponse(
+                self._cimi_request('PATCH', '{0}/{1}'.format(resource_type, operation),
+                                   json=data_with_filter, headers={'bulk': 'yes'}))
+
     def add(self, resource_type, data):
         """ Add a CIMI resource to the specified resource_type (Collection)
 
@@ -562,7 +590,7 @@ class Api(object):
         :param      data: The data to serialize into JSON
         :type       data: dict
 
-        :return:    A CimiResponse object which should contain the attributes 'status', 'resource-id' and 'message'
+        :return:    A CimiResponse object which should contain the attributes 'status', 'resource-id', 'message' and 'location'
         :rtype:     CimiResponse
         """
         operation_href = self._cimi_find_operation_href(resource, operation)
